/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define WWDG_IRQHandler wwdg_isr
#define PVD_PVM_IRQHandler pvd_pvm_isr
#define RTC_IRQHandler rtc_isr
#define RTC_S_IRQHandler rtc_s_isr
#define TAMP_IRQHandler tamp_isr
#define RAMCFG_IRQHandler ramcfg_isr
#define FLASH_IRQHandler flash_isr
#define FLASH_S_IRQHandler flash_s_isr
#define GTZC_IRQHandler gtzc_isr
#define RCC_IRQHandler rcc_isr
#define RCC_S_IRQHandler rcc_s_isr
#define EXTI0_IRQHandler exti0_isr
#define EXTI1_IRQHandler exti1_isr
#define EXTI2_IRQHandler exti2_isr
#define EXTI3_IRQHandler exti3_isr
#define EXTI4_IRQHandler exti4_isr
#define EXTI5_IRQHandler exti5_isr
#define EXTI6_IRQHandler exti6_isr
#define EXTI7_IRQHandler exti7_isr
#define EXTI8_IRQHandler exti8_isr
#define EXTI9_IRQHandler exti9_isr
#define EXTI10_IRQHandler exti10_isr
#define EXTI11_IRQHandler exti11_isr
#define EXTI12_IRQHandler exti12_isr
#define EXTI13_IRQHandler exti13_isr
#define EXTI14_IRQHandler exti14_isr
#define EXTI15_IRQHandler exti15_isr
#define IWDG_IRQHandler iwdg_isr
#define SAES_IRQHandler saes_isr
#define GPDMA1_CH0_IRQHandler gpdma1_ch0_isr
#define GPDMA1_CH1_IRQHandler gpdma1_ch1_isr
#define GPDMA1_CH2_IRQHandler gpdma1_ch2_isr
#define GPDMA1_CH3_IRQHandler gpdma1_ch3_isr
#define GPDMA1_CH4_IRQHandler gpdma1_ch4_isr
#define GPDMA1_CH5_IRQHandler gpdma1_ch5_isr
#define GPDMA1_CH6_IRQHandler gpdma1_ch6_isr
#define GPDMA1_CH7_IRQHandler gpdma1_ch7_isr
#define ADC12_IRQHandler adc12_isr
#define DAC1_IRQHandler dac1_isr
#define FDCAN1_IT0_IRQHandler fdcan1_it0_isr
#define FDCAN1_IT1_IRQHandler fdcan1_it1_isr
#define TIM1_BRK_IRQHandler tim1_brk_isr
#define TIM1_UP_IRQHandler tim1_up_isr
#define TIM1_TRG_COM_IRQHandler tim1_trg_com_isr
#define TIM1_CC_IRQHandler tim1_cc_isr
#define TIM2_IRQHandler tim2_isr
#define TIM3_IRQHandler tim3_isr
#define TIM4_IRQHandler tim4_isr
#define TIM5_IRQHandler tim5_isr
#define TIM6_IRQHandler tim6_isr
#define TIM7_IRQHandler tim7_isr
#define TIM8_BRK_IRQHandler tim8_brk_isr
#define TIM8_UP_IRQHandler tim8_up_isr
#define TIM8_TRG_COML_IRQHandler tim8_trg_coml_isr
#define TIM8_CC_IRQHandler tim8_cc_isr
#define I2C1_EV_IRQHandler i2c1_ev_isr
#define I2C1_ER_IRQHandler i2c1_er_isr
#define I2C2_EV_IRQHandler i2c2_ev_isr
#define I2C2_ER_IRQHandler i2c2_er_isr
#define SPI1_IRQHandler spi1_isr
#define SPI2_IRQHandler spi2_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define USART3_IRQHandler usart3_isr
#define UART4_IRQHandler uart4_isr
#define UART5_IRQHandler uart5_isr
#define LPUART1_IRQHandler lpuart1_isr
#define LPTIM1_IRQHandler lptim1_isr
#define LPTIM2_IRQHandler lptim2_isr
#define TIM15_IRQHandler tim15_isr
#define TIM16_IRQHandler tim16_isr
#define TIM17_IRQHandler tim17_isr
#define COMP_IRQHandler comp_isr
#define USB_IRQHandler usb_isr
#define CRS_IRQHandler crs_isr
#define FMC_IRQHandler fmc_isr
#define OCTOSPI1_IRQHandler octospi1_isr
#define PWR_S3WU_IRQHandler pwr_s3wu_isr
#define SDMMC1_IRQHandler sdmmc1_isr
#define SDMMC2_IRQHandler sdmmc2_isr
#define GPDMA1_CH8_IRQHandler gpdma1_ch8_isr
#define GPDMA1_CH9_IRQHandler gpdma1_ch9_isr
#define GPDMA1_CH10_IRQHandler gpdma1_ch10_isr
#define GPDMA1_CH11_IRQHandler gpdma1_ch11_isr
#define GPDMA1_CH12_IRQHandler gpdma1_ch12_isr
#define GPDMA1_CH13_IRQHandler gpdma1_ch13_isr
#define GPDMA1_CH14_IRQHandler gpdma1_ch14_isr
#define GPDMA1_CH15_IRQHandler gpdma1_ch15_isr
#define I2C3_EV_IRQHandler i2c3_ev_isr
#define I2C3_ER_IRQHandler i2c3_er_isr
#define SAI1_IRQHandler sai1_isr
#define SAI2_IRQHandler sai2_isr
#define TSC_IRQHandler tsc_isr
#define AES_IRQHandler aes_isr
#define RNG_IRQHandler rng_isr
#define FPU_IRQHandler fpu_isr
#define HASH_IRQHandler hash_isr
#define PKA_IRQHandler pka_isr
#define LPTIM3_IRQHandler lptim3_isr
#define SPI3_IRQHandler spi3_isr
#define I2C4_ER_IRQHandler i2c4_er_isr
#define I2C4_EV_IRQHandler i2c4_ev_isr
#define MDF1_FLT0_IRQHandler mdf1_flt0_isr
#define MDF1_FLT1_IRQHandler mdf1_flt1_isr
#define MDF1_FLT2_IRQHandler mdf1_flt2_isr
#define MDF1_FLT3_IRQHandler mdf1_flt3_isr
#define UCPD1_IRQHandler ucpd1_isr
#define ICACHE_IRQHandler icache_isr
#define OTFDEC1_IRQHandler otfdec1_isr
#define OTFDEC2_IRQHandler otfdec2_isr
#define LPTIM4_IRQHandler lptim4_isr
#define DCACHE1_IRQHandler dcache1_isr
#define ADF1_FLT0_IRQHandler adf1_flt0_isr
#define ADC4_IRQHandler adc4_isr
#define LPDMA1_CH0_IRQHandler lpdma1_ch0_isr
#define LPDMA1_CH1_IRQHandler lpdma1_ch1_isr
#define LPDMA1_CH2_IRQHandler lpdma1_ch2_isr
#define LPDMA1_CH3_IRQHandler lpdma1_ch3_isr
#define DMA2D_IRQHandler dma2d_isr
#define DCMI_PSSI_IRQHandler dcmi_pssi_isr
#define OCTOSPI2_IRQHandler octospi2_isr
#define MDF1_FLT4_IRQHandler mdf1_flt4_isr
#define MDF1_FLT5_IRQHandler mdf1_flt5_isr
#define CORDIC_IRQHandler cordic_isr
#define FMAC_IRQHandler fmac_isr
#define LSECSS_IRQHandler lsecss_isr
#define USART6_IRQHandler usart6_isr
#define I2C5_ER_IRQHandler i2c5_er_isr
#define I2C5_EV_IRQHandler i2c5_ev_isr
#define I2C6_ER_IRQHandler i2c6_er_isr
#define I2C6_EV_IRQHandler i2c6_ev_isr
#define HSPI1_IRQHandler hspi1_isr
#define GPU2D_IRQ_IRQHandler gpu2d_irq_isr
#define GPU2D_IRQSYS_IRQHandler gpu2d_irqsys_isr
#define GFXMMU_IRQHandler gfxmmu_isr
#define LCD_TFT_IRQHandler lcd_tft_isr
#define LCD_TFT_ERR_IRQHandler lcd_tft_err_isr
#define DSIHOST_IRQHandler dsihost_isr
#define DCACHE2_IRQHandler dcache2_isr
#define GFXTIM_IRQHandler gfxtim_isr
#define JPEG_IRQHandler jpeg_isr
