/* This file is part of the libopencm3 project.
 *
 * It was generated by the irq2nvic_h script.
 *
 * These definitions bend every interrupt handler that is defined CMSIS style
 * to the weak symbol exported by libopencm3.
 */

#define SUPC_IRQHandler supc_isr
#define RSTC_IRQHandler rstc_isr
#define RTC_IRQHandler rtc_isr
#define RTT_IRQHandler rtt_isr
#define WDT_IRQHandler wdt_isr
#define PMC_IRQHandler pmc_isr
#define EEFC0_IRQHandler eefc0_isr
#define EEFC1_IRQHandler eefc1_isr
#define UART_IRQHandler uart_isr
#define SMC_IRQHandler smc_isr
#define PIOA_IRQHandler pioa_isr
#define PIOB_IRQHandler piob_isr
#define PIOC_IRQHandler pioc_isr
#define USART0_IRQHandler usart0_isr
#define USART1_IRQHandler usart1_isr
#define USART2_IRQHandler usart2_isr
#define USART3_IRQHandler usart3_isr
#define HSMCI_IRQHandler hsmci_isr
#define TWI0_IRQHandler twi0_isr
#define TWI1_IRQHandler twi1_isr
#define SPI_IRQHandler spi_isr
#define SSC_IRQHandler ssc_isr
#define TC0_IRQHandler tc0_isr
#define TC1_IRQHandler tc1_isr
#define TC2_IRQHandler tc2_isr
#define PWM_IRQHandler pwm_isr
#define ADC12B_IRQHandler adc12b_isr
#define ADC_IRQHandler adc_isr
#define DMAC_IRQHandler dmac_isr
#define UDPHS_IRQHandler udphs_isr
